@@ -2,9 +2,11 @@ package main
 
 import (
 	"context"
+	"database/sql"
+	"fmt"
 	"net"
+	"os"
 
-	"github.com/operator-framework/operator-registry/pkg/lib/log"
 	"github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 	"google.golang.org/grpc"
@@ -12,7 +14,10 @@ import (
 
 	"github.com/operator-framework/operator-registry/pkg/api"
 	health "github.com/operator-framework/operator-registry/pkg/api/grpc_health_v1"
-	"github.com/operator-framework/operator-registry/pkg/registry"
+	"github.com/operator-framework/operator-registry/pkg/lib/dns"
+	"github.com/operator-framework/operator-registry/pkg/lib/graceful"
+	"github.com/operator-framework/operator-registry/pkg/lib/log"
+	"github.com/operator-framework/operator-registry/pkg/lib/tmp"
 	"github.com/operator-framework/operator-registry/pkg/server"
 	"github.com/operator-framework/operator-registry/pkg/sqlite"
 )
@@ -36,6 +41,7 @@ func init() {
 	rootCmd.Flags().StringP("database", "d", "bundles.db", "relative path to sqlite db")
 	rootCmd.Flags().StringP("port", "p", "50051", "port number to serve on")
 	rootCmd.Flags().StringP("termination-log", "t", "/dev/termination-log", "path to a container termination log file")
+	rootCmd.Flags().Bool("skip-migrate", false, "do  not attempt to migrate to the latest db revision when starting")
 	if err := rootCmd.Flags().MarkHidden("debug"); err != nil {
 		logrus.Panic(err.Error())
 	}
@@ -57,6 +63,10 @@ func runCmdFunc(cmd *cobra.Command, args []string) error {
 	if err != nil {
 		logrus.WithError(err).Warn("unable to set termination log path")
 	}
+	// Ensure there is a default nsswitch config
+	if err := dns.EnsureNsswitch(); err != nil {
+		return err
+	}
 	dbName, err := cmd.Flags().GetString("database")
 	if err != nil {
 		return err
@@ -69,15 +79,25 @@ func runCmdFunc(cmd *cobra.Command, args []string) error {
 
 	logger := logrus.WithFields(logrus.Fields{"database": dbName, "port": port})
 
-	var store registry.Query
-	store, err = sqlite.NewSQLLiteQuerier(dbName)
+	// make a writable copy of the db for migrations
+	tmpdb, err := tmp.CopyTmpDB(dbName)
 	if err != nil {
-		logger.WithError(err).Warnf("failed to load db")
+		return err
 	}
-	if store == nil {
-		store = registry.NewEmptyQuerier()
+	defer os.Remove(tmpdb)
+
+	db, err := sql.Open("sqlite3", tmpdb)
+	if err != nil {
+		return err
 	}
 
+	// migrate to the latest version
+	if err := migrate(cmd, db); err != nil {
+		logger.WithError(err).Warnf("couldn't migrate db")
+	}
+
+	store := sqlite.NewSQLLiteQuerierFromDb(db)
+
 	// sanity check that the db is available
 	tables, err := store.ListTables(context.TODO())
 	if err != nil {
@@ -97,9 +117,30 @@ func runCmdFunc(cmd *cobra.Command, args []string) error {
 	health.RegisterHealthServer(s, server.NewHealthServer())
 	reflection.Register(s)
 	logger.Info("serving registry")
-	if err := s.Serve(lis); err != nil {
-		logger.Fatalf("failed to serve: %s", err)
+
+	return graceful.Shutdown(logger, func() error {
+		return s.Serve(lis)
+	}, func() {
+		s.GracefulStop()
+	})
+}
+
+func migrate(cmd *cobra.Command, db *sql.DB) error {
+	shouldSkipMigrate, err := cmd.Flags().GetBool("skip-migrate")
+	if err != nil {
+		return err
+	}
+	if shouldSkipMigrate {
+		return nil
 	}
 
-	return nil
+	migrator, err := sqlite.NewSQLLiteMigrator(db)
+	if err != nil {
+		return err
+	}
+	if migrator == nil {
+		return fmt.Errorf("failed to load migrator")
+	}
+
+	return migrator.Migrate(context.TODO())
 }
