@@ -1231,3 +1231,105 @@ func (s *SQLQuerier) GetBundlePathIfExists(ctx context.Context, bundleName strin
 
 	return
 }
+
+// ListRegistryBundles returns a set of registry bundles.
+// The set can be filtered by package by setting the given context's 'package' key to a desired package name.
+// e.g.
+// ctx := ContextWithPackage(context.TODO(), "etcd")
+// bundles, err := querier.ListRegistryBundles(ctx)
+// // ...
+func (s *SQLQuerier) ListRegistryBundles(ctx context.Context) ([]*registry.Bundle, error) {
+	listBundlesQuery := `
+	SELECT DISTINCT operatorbundle.name, operatorbundle.version, operatorbundle.bundle, channel_entry.package_name
+	FROM operatorbundle
+	LEFT OUTER JOIN channel_entry ON operatorbundle.name = channel_entry.operatorbundle_name`
+
+	var (
+		err  error
+		rows RowScanner
+	)
+	if pkg, ok := registry.PackageFromContext(ctx); ok {
+		listBundlesQuery += " WHERE channel_entry.package_name=?"
+		rows, err = s.db.QueryContext(ctx, listBundlesQuery, pkg)
+	} else {
+		rows, err = s.db.QueryContext(ctx, listBundlesQuery)
+	}
+	if err != nil {
+		return nil, err
+	}
+	defer rows.Close()
+
+	var bundles []*registry.Bundle
+	for rows.Next() {
+		var (
+			bundleName    sql.NullString
+			bundleVersion sql.NullString
+			bundle        sql.NullString
+			packageName   sql.NullString
+		)
+		if err := rows.Scan(&bundleName, &bundleVersion, &bundle, &packageName); err != nil {
+			return nil, err
+		}
+
+		switch {
+		case !bundleName.Valid:
+			return nil, fmt.Errorf("bundle name column corrupted")
+		case !bundleVersion.Valid:
+			// Version field is currently nullable
+		case !bundle.Valid:
+			// Bundle field is currently nullable
+		case !packageName.Valid:
+			return nil, fmt.Errorf("package name column corrupted")
+		}
+
+		// Allow the channel_entry table to be authoritative
+		channels, err := s.listBundleChannels(ctx, bundleName.String)
+		if err != nil {
+			return nil, fmt.Errorf("unable to list channels for bundle %s: %s", bundleName.String, err)
+		}
+
+		defaultChannel, err := s.GetDefaultChannelForPackage(ctx, packageName.String)
+		if err != nil {
+			return nil, fmt.Errorf("unable to get default channel for package %s: %s", packageName.String, err)
+		}
+
+		b, err := registry.NewBundleFromStrings(bundleName.String, bundleVersion.String, packageName.String, defaultChannel, strings.Join(channels, ","), bundle.String)
+		if err != nil {
+			return nil, fmt.Errorf("unable to unmarshal bundle %s from database: %s", bundleName.String, err)
+		}
+
+		bundles = append(bundles, b)
+	}
+
+	return bundles, nil
+}
+
+func (s *SQLQuerier) listBundleChannels(ctx context.Context, bundleName string) ([]string, error) {
+	listBundleChannelsQuery := `
+	SELECT DISTINCT channel_entry.channel_name
+	FROM channel_entry
+	INNER JOIN operatorbundle ON channel_entry.operatorbundle_name = operatorbundle.name
+	WHERE operatorbundle.name = ?`
+
+	rows, err := s.db.QueryContext(ctx, listBundleChannelsQuery, bundleName)
+	if err != nil {
+		return nil, err
+	}
+	defer rows.Close()
+
+	var channels []string
+	for rows.Next() {
+		var channel sql.NullString
+		if err := rows.Scan(&channel); err != nil {
+			return nil, err
+		}
+
+		if !channel.Valid {
+			return nil, fmt.Errorf("channel name column corrupt for bundle %s", bundleName)
+		}
+
+		channels = append(channels, channel.String)
+	}
+
+	return channels, nil
+}
